unit FntNmSet;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls,
  Forms, Dialogs, DsgnIntf, TypInfo;

type
  TSettingType = (ForAll,ForTypes,ForNames);

  TFontNameSetter = class(TComponent)
  private
    FFontName : TFontName;
    FSettingType : TSettingType;
    FNames : TStrings;
    FTypes : TStrings;
  protected
    procedure SetNames(Value: TStrings);
    procedure SetTypes(Value: TStrings);
    function InTypes(Comp:TComponent): Boolean;
    function InNames(Comp:TComponent): Boolean;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure SetFontName;
  published
    property SettingType : TSettingType read FSettingType write FSettingType default ForAll;
    property Names : TStrings read FNames write SetNames;{stored (FSettingType = ForNames)}
    property Types : TStrings read FTypes write SetTypes;
    property FontName : TFontName read FFontName write FFontName;
  end;

implementation

constructor TFontNameSetter.Create(AOwner: TComponent);
var
  PropInfo: PPropInfo;
  CompFont : TFont;
begin
 inherited Create(AOwner);
 PropInfo := GetPropInfo(AOwner.ClassInfo, 'Font');
 if PropInfo = nil then exit;
 if not (AOwner is TForm) then exit;
 FFontName := (AOwner As TForm).Font.Name;
 FNames := TStringList.Create;
 FTypes := TStringList.Create;
end;

destructor TFontNameSetter.Destroy;
begin
 inherited Destroy;
 FNames.Free;
 FTypes.Free;
end;

procedure TFontNameSetter.SetFontName;
var
  I: Integer;
  PropInfo: PPropInfo;
  CompFont : TFont;
begin
 with (Owner as TComponent) do
  begin
   for I := 0 to ComponentCount-1 do
   begin
     PropInfo := GetPropInfo(Components[I].ClassInfo, 'Font');
     if PropInfo = nil then continue;
     case SettingType of
      ForTypes: if not InTypes(Components[I]) then continue;
      ForNames: if not InNames(Components[I]) then continue;
     end;
     CompFont := TFont(GetOrdProp(Components[I], PropInfo));
     CompFont.Name := FFontName;
   end;
   PropInfo := GetPropInfo(Self.Owner.ClassInfo, 'Font');
   case SettingType of
      ForTypes: if not InTypes(Self.Owner) then exit;
      ForNames: if not InNames(Self.Owner) then exit;
     end;
   CompFont := TFont(GetOrdProp(Self.Owner, PropInfo));
   CompFont.Name := FFontName;
  end;
end;

procedure TFontNameSetter.SetTypes(Value: TStrings);
begin
  if FTypes.Text <> Value.Text then
  begin
    FTypes.BeginUpdate;
    try
      FTypes.Assign(Value);
    finally
      FTypes.EndUpdate;
    end;
  end;
end;

procedure TFontNameSetter.SetNames(Value: TStrings);
begin
  if FNames.Text <> Value.Text then
  begin
    FNames.BeginUpdate;
    try
      FNames.Assign(Value);
    finally
      FNames.EndUpdate;
    end;
  end;
end;

function TFontNameSetter.InTypes(Comp:TComponent): Boolean;
begin
 Result := (Types.IndexOf(Comp.ClassName)<> -1)
end;

function TFontNameSetter.InNames(Comp:TComponent): Boolean;
begin
 Result := (Names.IndexOf(Comp.Name)<> -1);
end;

end.
